"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createResourceController = void 0;
const future_1 = require("../future");
function createResourceController(task, resource) {
    let resourceTask;
    let initTask;
    let { scope } = task.options;
    let { produce, future } = (0, future_1.createFuture)();
    function start() {
        var _a;
        if (!scope) {
            throw new Error('cannot spawn resource in task which has no resource scope');
        }
        let name = resource.name || ((_a = resource.labels) === null || _a === void 0 ? void 0 : _a.name) || 'resource';
        let labels = resource.labels || {};
        resourceTask = scope.run(undefined, { type: 'resource', labels: { ...labels, name } });
        initTask = resourceTask.run((task) => resource.init(resourceTask, task), {
            yieldScope: resourceTask,
            labels: { name: 'init' },
            ignoreError: true,
        });
        initTask.consume((value) => {
            if (value.state !== 'completed') {
                resourceTask.halt();
            }
            produce(value);
        });
    }
    function halt() {
        resourceTask === null || resourceTask === void 0 ? void 0 : resourceTask.halt();
    }
    return {
        type: 'resource constructor',
        start,
        halt,
        future,
        get resourceTask() {
            return resourceTask;
        },
        operation: resource
    };
}
exports.createResourceController = createResourceController;
//# sourceMappingURL=resource-controller.js.map