"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.timeout = void 0;
const sleep_1 = require("./sleep");
const labels_1 = require("../labels");
class TimeoutError extends Error {
    constructor() {
        super(...arguments);
        this.name = "TimeoutError";
    }
}
/**
 * Throw an error after the given amount of milliseconds. See also {@link withTimeout}.
 *
 * ### Example
 *
 * ```typescript
 * import { main, spawn, timeout } from 'effection';
 *
 * main(function*() {
 *   yield spawn(timeout(2000));
 *   yield fetch('http://google.com');
 * });
 * ```
 *
 * @param duration the timeout duration in milliseconds
 */
function timeout(duration) {
    return (0, labels_1.withLabels)(function* () {
        yield (0, sleep_1.sleep)(duration);
        throw new TimeoutError(`timed out after ${duration}ms`);
    }, { name: 'timeout', duration });
}
exports.timeout = timeout;
//# sourceMappingURL=timeout.js.map