"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.withTimeout = void 0;
const timeout_1 = require("./timeout");
const race_1 = require("./race");
const labels_1 = require("../labels");
/**
 * Runs the given operation and throws an error if it takes more than the given
 * number of milliseconds. See also {@link timeout}.
 *
 * ### Example
 *
 * ```typescript
 * import { main, spawn, withTimeout } from 'effection';
 *
 * main(function*() {
 *   yield withTimeout(2000, fetch('http://google.com'));
 * });
 * ```
 *
 * @param duration the timeout duration in milliseconds
 */
function withTimeout(duration, operation) {
    return (0, labels_1.setLabels)((0, race_1.race)([(0, timeout_1.timeout)(duration), operation]), { name: 'withTimeout', duration });
}
exports.withTimeout = withTimeout;
//# sourceMappingURL=with-timeout.js.map